# copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
# this program is free software: please refer to the BSD type license enclosed in this package

set rcsId {$Id: datagraf.tcl,v 1.23 1998/03/10 22:35:01 jfontain Exp $}

class dataGraph {

    proc dataGraph {this parentPath args} composite {
        [new graph $parentPath -title {} -topmargin 3 -bufferelements 0 -plotborderwidth 1] $args
    } blt2DViewer {$widget::($this,path)} {
        $widget::($this,path) xaxis configure -tickfont {Helvetica -10} -title {} -rotate 90 -command dataGraph::axisTime
        composite::complete $this
    }

    proc ~dataGraph {this} {}

    proc iconData {} {
        return {
            R0lGODdhKAAoAKUAAHt5e87PzgAAANbX1v///zFhITFhGJTXa633e6X3e5znc6Xvc3u+Wpznazk4
            OZTfa4zPY3NJAPffSufHOda2Mc6mKVIIY2sYexBhe4wQpa3X54S+zr0o1qUYvbUYzrUg1lKWrb0Y
            3t44/60Yzt5B/+dJ/+dR/9YY984Y7+dZ/+dh/94o/+9x/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
            AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAKAAoAAAG/kCAcEgsGo/IYQAg
            aDqf0Kh02lwKAthsYIDlbrVer3Y8ZgYI6DQaoG6z2+n3Gm09wwlyeN6tflsHd3iBgoF5fmZDBYpJ
            AIoGBUaPj0Z1aAUHmAWEapeYB5ppBQgICQqghwKABJ2ZeYoLCwoLpYqKpbAMkISVqwyeC6CrsAoK
            DAzDCg3FC8aQbKhLQgW+BwsNtdUNzdMMxb7bDuFEf5zGyp4H2osOisbbAA5y0G3cD98QuUPxjbX6
            fXRMVHHi5s7ZHCNz4gC8gqTWIkYQAfCCs2/Iv0IXyd2JwJEjH4wKCUxMw1GCSZMRLt4xtDDakAgm
            J8iUKSGCkY42GY2EKZNC/oWfFCbUTAmg5EmPLEUGJCmhZ4WOFYKihNl05tSMZtDwnOAzZ9GfNJv6
            /BlVaM2Qf4REoMD2abi3HKOy7Qo1aNeKErMSiEDWpgM3fPsSCfzT5i69ezkK+Qu4oyGcQg5f6ROZ
            sZ54K/cltEKls2cpViyIvoCh9IXTqFOXNp2a9OrWF6xkmI1Bg20Ms3Pnrm1bA+7dvTdguMChOAcr
            HTx8wLChue8P0KGX1tDc+era1UEMDxFChAgrIzyMwACi+nPptpuDKL+hN/UN64dfCEGiPvgR0tm3
            X/0+Pnn98GknH3clFAgeByEg+J9z6cHHWmnr+Ycad/XZxwR3GIawYHUOXsJ23YQYmiCiCVZwhyCC
            Gv4X4YAYwjbiiyOWeAIKM6KAQgiuvXZBCjz26OOPQFpRo41E4phaBiGooOSSTDbppJAoOCnllFQ2
            acUKK7Cg5ZZcdunll18uQcaYZJZZZhAAOw==
        }
    }

    proc options {this} {
        # force size values
        return [list\
            [list -height height Height 200]\
            [list -interval interval Interval 5]\
            [list -samples samples Samples 101 101]\
            [list -width width Width 300]\
        ]
    }

    foreach option {-height -width} {
        proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
    }

    proc set-interval {this value} {
        xAxisUpdate $this [clock seconds]
    }

    proc set-samples {this value} {                                                                     ;# stored at composite level
        if {$composite::($this,complete)} {
            error {option -samples cannot be set dynamically}
        }
    }

    proc xAxisUpdate {this currentTime} { ;### calculate steps sizes according to interval ###
        $widget::($this,path) xaxis configure\
            -min [expr {$currentTime-($composite::($this,-samples)*$composite::($this,-interval))}] -max $currentTime\
            -stepsize 60 -subdivisions 6
    }

    proc update {this args} {                                    ;# update display using cells data. ignore eventual trace arguments
        xAxisUpdate $this [set seconds [clock seconds]]
        foreach element $blt2DViewer::($this,elements) {
            if {[catch {set $blt2DViewer::($this,data,$element)} value]} {
                element::update $element $seconds {}                                                        ;# data no longer exists
            } else {
                element::update $element $seconds $value
            }
        }
    }

    proc axisTime {path value} {
        return [clock format $value -format %T]
    }

    proc newElement {this path args} {                                                          ;# invoked from 2D viewer base class
        return [eval new element $path $args]
    }
}

class dataGraph {
    class element {

        set (intervals) 100                                                                       ;### should be parameterizable ###

        proc element {this path args} switched {$args} {
            $path element line $this -label {}                                        ;# use object identifier as element identifier

            ### axis data vectors (should be a namespace variable when blt is fixed) ###
            global [set xVector ${this}xData] [set yVector ${this}yData]
            global $xVector  ;### should be a variable but it causes a "called Tcl_FindHashEntry on deleted table" error on exit ###
            set dots [expr {[set (intervals)]+1}]
            blt::vector ${xVector}($dots)
            $path element configure $this -xdata $xVector

            set yVector ${this}yData                                                                           ;# y axis data vector
            global $yVector
            blt::vector ${yVector}($dots)
            $path element configure $this -ydata $yVector

            set ($this,path) $path
            switched::complete $this
        }

        proc ~element {this} {
            global [set xVector ${this}xData] [set yVector ${this}yData]
            unset $xVector $yVector
            [set ($this,path)] element delete $this
            if {[string length $switched::($this,-deletecommand)]>0} {
                uplevel #0 $switched::($this,-deletecommand)                                ;# always invoke command at global level
            }
        }

        proc options {this} {
            return [list\
                [list -color black black]\
                [list -deletecommand {} {}]\
                [list -label {} {}]\
            ]
        }

        foreach option {-color -label} {
            proc set$option {this value} "\[set (\$this,path)\] element configure \$this $option \$value"
        }

        proc set-deletecommand {this value} {}                                                   ;# data is stored at switched level

        proc update {this x y} {
            global [set xVector ${this}xData] [set yVector ${this}yData]

            if {[string length $y]==0} {
                set y 0                                                              ;# too bad BLT graph cannot handle empty values
            }
            $xVector delete 0
            $xVector append $x
            $yVector delete 0                                                         ;# achieve scrolling by deleting first element
            $yVector append $y                                                                            ;# and appending new value
        }
    }
}
